@@ -5,11 +5,11 @@ then
     POEZIO_VENV="poezio-venv"
 fi
 
-if [ -d .git ]
+if [ -e .git ]
 then
     args=$(git show --format='%h %ci' | head -n1)
 else
-    args="0.9-dev"
+    args="0.13.1-dev"
 fi
 
 if [ -e "$POEZIO_VENV" ]
@@ -24,6 +24,6 @@ else
     PYTHON3=python3
 fi
 
-$PYTHON3 -c 'import sys;(print("Python 3.5 or newer is required") and exit(1)) if sys.version_info < (3, 5) else exit(0)' || exit 1
-exec "$PYTHON3" -m poezio -v "$args" "$@"
+$PYTHON3 -c 'import sys;(print("Python 3.7 or newer is required") and exit(1)) if sys.version_info < (3, 7) else exit(0)' || exit 1
+exec "$PYTHON3" -m poezio --custom-version "$args" "$@"
 
