@@ -13,6 +13,7 @@ Configuration
 
 from poezio.plugin import BasePlugin
 
+
 class Plugin(BasePlugin):
     def init(self):
         self.api.add_event_handler('muc_say', self.advert)
@@ -20,4 +21,5 @@ class Plugin(BasePlugin):
         self.api.add_event_handler('private_say', self.advert)
 
     def advert(self, msg, tab):
-        msg['body'] = "%s\n\n%s" % (msg['body'], self.config.get("ad", "Sent from poezio"))
+        msg['body'] = "%s\n\n%s" % (msg['body'],
+                                    self.config.get("ad", "Sent from poezio"))
