@@ -13,24 +13,28 @@ import pipes
 import sys
 import shutil
 import time
+import uuid
 from collections import defaultdict
-from typing import Callable, Dict, List, Optional, Tuple, Type
+from typing import Callable, Dict, List, Optional, Set, Tuple, Type
+from xml.etree import ElementTree as ET
+from functools import partial
 
-from slixmpp import JID
+from slixmpp import JID, InvalidJID
 from slixmpp.util import FileSystemPerJidCache
 from slixmpp.xmlstream.handler import Callback
+from slixmpp.exceptions import IqError, IqTimeout
 
 from poezio import connection
 from poezio import decorators
 from poezio import events
 from poezio import multiuserchat as muc
 from poezio import tabs
+from poezio import mam
 from poezio import theming
 from poezio import timed_events
 from poezio import windows
 
 from poezio.bookmarks import BookmarkList
-from poezio.common import safeJID
 from poezio.config import config, firstrun
 from poezio.contact import Contact, Resource
 from poezio.daemon import Executor
@@ -41,6 +45,7 @@ from poezio.roster import roster
 from poezio.size_manager import SizeManager
 from poezio.user import User
 from poezio.text_buffer import TextBuffer
+from poezio.timed_events import DelayedEvent
 from poezio.theming import get_theme
 from poezio import keyboard, xdg
 
@@ -48,8 +53,15 @@ from poezio.core.completions import CompletionCore
 from poezio.core.tabs import Tabs
 from poezio.core.commands import CommandCore
 from poezio.core.handlers import HandlerCore
-from poezio.core.structs import POSSIBLE_SHOW, DEPRECATED_ERRORS, \
-        ERROR_AND_STATUS_CODES, Command, Status
+from poezio.core.structs import (
+    Command,
+    Status,
+    DEPRECATED_ERRORS,
+    ERROR_AND_STATUS_CODES,
+    POSSIBLE_SHOW,
+)
+
+from poezio.ui.types import Message, InfoMessage
 
 log = logging.getLogger(__name__)
 
@@ -154,10 +166,12 @@ class Core:
             "KEY_F(5)": self.rotate_rooms_left,
             "^P": self.rotate_rooms_left,
             "M-[-D": self.rotate_rooms_left,
+            "M-[1;3D": self.rotate_rooms_left,
             'kLFT3': self.rotate_rooms_left,
             "KEY_F(6)": self.rotate_rooms_right,
             "^N": self.rotate_rooms_right,
             "M-[-C": self.rotate_rooms_right,
+            "M-[1;3C": self.rotate_rooms_right,
             'kRIT3': self.rotate_rooms_right,
             "KEY_F(4)": self.toggle_left_pane,
             "KEY_F(7)": self.shrink_information_win,
@@ -195,6 +209,7 @@ class Core:
             '_show_plugins': self.command.plugins,
             '_show_xmltab': self.command.xml_tab,
             '_toggle_pane': self.toggle_left_pane,
+            "_go_to_room_name": self.go_to_room_name,
             ###### status actions ######
             '_available': lambda: self.command.status('available'),
             '_away': lambda: self.command.status('away'),
@@ -220,6 +235,7 @@ class Core:
             ('connected', self.handler.on_connected),
             ('connection_failed', self.handler.on_failed_connection),
             ('disconnected', self.handler.on_disconnected),
+            ('reconnect_delay', self.handler.on_reconnect_delay),
             ('failed_all_auth', self.handler.on_failed_all_auth),
             ('got_offline', self.handler.on_got_offline),
             ('got_online', self.handler.on_got_online),
@@ -233,6 +249,7 @@ class Core:
             ('groupchat_subject', self.handler.on_groupchat_subject),
             ('http_confirm', self.handler.http_confirm),
             ('message', self.handler.on_message),
+            ('message_encryption', self.handler.on_encrypted_message),
             ('message_error', self.handler.on_error_message),
             ('message_xform', self.handler.on_data_form),
             ('no_auth', self.handler.on_no_auth),
@@ -249,6 +266,9 @@ class Core:
             ('roster_update', self.handler.on_roster_update),
             ('session_start', self.handler.on_session_start),
             ('session_start', self.handler.on_session_start_features),
+            ('session_end', self.handler.on_session_end),
+            ('sm_failed', self.handler.on_session_end),
+            ('session_resumed', self.handler.on_session_resumed),
             ('ssl_cert', self.handler.validate_ssl),
             ('ssl_invalid_chain', self.handler.ssl_invalid_chain),
             ('stream_error', self.handler.on_stream_error),
@@ -506,10 +526,10 @@ class Core:
         plugins = config.get('plugins_autoload')
         if ':' in plugins:
             for plugin in plugins.split(':'):
-                self.plugin_manager.load(plugin)
+                self.plugin_manager.load(plugin, unload_first=False)
         else:
             for plugin in plugins.split():
-                self.plugin_manager.load(plugin)
+                self.plugin_manager.load(plugin, unload_first=False)
         self.plugins_autoloaded = True
 
     def start(self):
@@ -523,6 +543,12 @@ class Core:
         default_tab.on_gain_focus()
         self.tabs.append(default_tab)
         self.information('Welcome to poezio!', 'Info')
+        if curses.COLORS < 256:
+            self.information(
+                'Your terminal does not appear to support 256 colors, the UI'
+                ' colors will probably be ugly',
+                'Error',
+            )
         if firstrun:
             self.information(
                 'It seems that it is the first time you start poezio.\n'
@@ -628,13 +654,13 @@ class Core:
         """
         if isinstance(roster_row, Contact):
             if not self.get_conversation_by_jid(roster_row.bare_jid, False):
-                self.open_conversation_window(roster_row.bare_jid)
+                self.open_conversation_window(JID(roster_row.bare_jid))
             else:
                 self.focus_tab_named(roster_row.bare_jid)
         if isinstance(roster_row, Resource):
             if not self.get_conversation_by_jid(
                     roster_row.jid, False, fallback_barejid=False):
-                self.open_conversation_window(roster_row.jid)
+                self.open_conversation_window(JID(roster_row.jid))
             else:
                 self.focus_tab_named(roster_row.jid)
         self.refresh_window()
@@ -776,24 +802,24 @@ class Core:
 
 ########################## TImed Events #######################################
 
-    def remove_timed_event(self, event):
+    def remove_timed_event(self, event: DelayedEvent) -> None:
         """Remove an existing timed event"""
         event.handler.cancel()
 
-    def add_timed_event(self, event):
+    def add_timed_event(self, event: DelayedEvent) -> None:
         """Add a new timed event"""
         event.handler = asyncio.get_event_loop().call_later(
             event.delay, event.callback, *event.args)
 
 ####################### XMPP-related actions ##################################
 
-    def get_status(self):
+    def get_status(self) -> str:
         """
         Get the last status that was previously set
         """
         return self.status
 
-    def set_status(self, pres: str, msg: str):
+    def set_status(self, pres: str, msg: str) -> None:
         """
         Set our current status so we can remember
         it and use it back when needed (for example to display it
@@ -819,23 +845,18 @@ class Core:
             return bm.nick
         return self.own_nick
 
-    def disconnect(self, msg='', reconnect=False):
+    def disconnect(self, msg: str = '', reconnect: bool = False) -> None:
         """
         Disconnect from remote server and correctly set the states of all
         parts of the client (for example, set the MucTabs as not joined, etc)
         """
         self.legitimate_disconnect = True
-        msg = msg or ''
-        for tab in self.get_tabs(tabs.MucTab):
-            tab.command_part(msg)
-        self.xmpp.disconnect()
         if reconnect:
-            # Add a one-time event to reconnect as soon as we are
-            # effectively disconnected
-            self.xmpp.add_event_handler(
-                'disconnected',
-                lambda event: self.xmpp.connect(),
-                disposable=True)
+            self.xmpp.reconnect(wait=0.0, reason=msg)
+        else:
+            for tab in self.get_tabs(tabs.MucTab):
+                tab.leave_room(msg)
+            self.xmpp.disconnect(reason=msg)
 
     def send_message(self, msg: str) -> bool:
         """
@@ -848,7 +869,7 @@ class Core:
         self.tabs.current_tab.command_say(msg)
         return True
 
-    def invite(self, jid: JID, room: JID, reason: Optional[str] = None):
+    def invite(self, jid: JID, room: JID, reason: Optional[str] = None) -> None:
         """
         Checks if the sender supports XEP-0249, then send an invitation,
         or a mediated one if it does not.
@@ -868,7 +889,94 @@ class Core:
         self.xmpp.plugin['xep_0030'].get_info(
             jid=jid, timeout=5, callback=callback)
 
-    def get_error_message(self, stanza, deprecated=False):
+    def _impromptu_room_form(self, room):
+        fields = [
+            ('hidden', 'FORM_TYPE', 'http://jabber.org/protocol/muc#roomconfig'),
+            ('boolean', 'muc#roomconfig_changesubject', True),
+            ('boolean', 'muc#roomconfig_allowinvites', True),
+            ('boolean', 'muc#roomconfig_persistent', True),
+            ('boolean', 'muc#roomconfig_membersonly', True),
+            ('boolean', 'muc#roomconfig_publicroom', False),
+            ('list-single', 'muc#roomconfig_whois', 'anyone'),
+            # MAM
+            ('boolean', 'muc#roomconfig_enablearchiving', True),  # Prosody
+            ('boolean', 'mam', True),  # Ejabberd community
+            ('boolean', 'muc#roomconfig_mam', True),  # Ejabberd saas
+        ]
+
+        form = self.xmpp['xep_0004'].make_form()
+        form['type'] = 'submit'
+        for field in fields:
+            form.add_field(
+                ftype=field[0],
+                var=field[1],
+                value=field[2],
+            )
+
+        iq = self.xmpp.Iq()
+        iq['type'] = 'set'
+        iq['to'] = room
+        query = ET.Element('{http://jabber.org/protocol/muc#owner}query')
+        query.append(form.xml)
+        iq.append(query)
+        return iq
+
+    async def impromptu(self, jids: Set[JID]) -> None:
+        """
+        Generates a new "Impromptu" room with a random localpart on the muc
+        component of the user who initiated the request. One the room is
+        created and the first user has joined, send invites for specified
+        contacts to join in.
+        """
+
+        results = await self.xmpp['xep_0030'].get_info_from_domain()
+
+        muc_from_identity = ''
+        for info in results:
+            for identity in info['disco_info']['identities']:
+                if identity[0] == 'conference' and identity[1] == 'text':
+                    muc_from_identity = info['from'].bare
+
+        # Use config.default_muc_service as muc component if available,
+        # otherwise find muc component by disco#items-ing the user domain.
+        # If not, give up
+        default_muc = config.get('default_muc_service', muc_from_identity)
+        if not default_muc:
+            self.information(
+                "Error finding a MUC service to join. If your server does not "
+                "provide one, set 'default_muc_service' manually to a MUC "
+                "service that allows room creation.",
+                'Error'
+            )
+            return
+
+        nick = self.own_nick
+        localpart = uuid.uuid4().hex
+        room_str = '{!s}@{!s}'.format(localpart, default_muc)
+        try:
+            room = JID(room_str)
+        except InvalidJID:
+            self.information(
+                'The generated XMPP address is invalid: {!s}'.format(room_str),
+                'Error'
+            )
+            return None
+
+        self.open_new_room(room, nick).join()
+        iq = self._impromptu_room_form(room)
+        try:
+            await iq.send()
+        except (IqError, IqTimeout):
+            self.information('Failed to configure impromptu room.', 'Info')
+            # TODO: destroy? leave room.
+            return None
+
+        self.information('Room %s created' % room, 'Info')
+
+        for jid in jids:
+            self.invite(jid, room)
+
+    def get_error_message(self, stanza, deprecated: bool = False):
         """
         Takes a stanza of the form <message type='error'><error/></message>
         and return a well formed string containing error information
@@ -908,7 +1016,7 @@ class Core:
 
 ### Tab getters ###
 
-    def get_tabs(self, cls: Type[tabs.Tab] = None):
+    def get_tabs(self, cls: Type[tabs.Tab] = None) -> List[tabs.Tab]:
         "Get all the tabs of a type"
         if cls is None:
             return self.tabs.get_tabs()
@@ -916,8 +1024,8 @@ class Core:
 
     def get_conversation_by_jid(self,
                                 jid: JID,
-                                create=True,
-                                fallback_barejid=True) -> tabs.ChatTab:
+                                create: bool = True,
+                                fallback_barejid: bool = True) -> Optional[tabs.ChatTab]:
         """
         From a JID, get the tab containing the conversation with it.
         If none already exist, and create is "True", we create it
@@ -926,7 +1034,7 @@ class Core:
         If fallback_barejid is True, then this method will seek other
         tabs with the same barejid, instead of searching only by fulljid.
         """
-        jid = safeJID(jid)
+        jid = JID(jid)
         # We first check if we have a static conversation opened
         # with this precise resource
         conversation = self.tabs.by_name_and_class(jid.full,
@@ -945,12 +1053,12 @@ class Core:
                     # nothing was found (and we lock it to the resource
                     # later)
                     conversation = self.open_conversation_window(
-                        jid.bare, False)
+                        JID(jid.bare), False)
                 else:
                     conversation = None
         return conversation
 
-    def add_tab(self, new_tab: tabs.Tab, focus=False):
+    def add_tab(self, new_tab: tabs.Tab, focus: bool = False) -> None:
         """
         Appends the new_tab in the tab list and
         focus it if focus==True
@@ -959,7 +1067,7 @@ class Core:
         if focus:
             self.tabs.set_current_tab(new_tab)
 
-    def insert_tab(self, old_pos: int, new_pos=99999) -> bool:
+    def insert_tab(self, old_pos: int, new_pos: int = 99999) -> bool:
         """
         Insert a tab at a position, changing the number of the following tabs
         returns False if it could not move the tab, True otherwise
@@ -969,25 +1077,25 @@ class Core:
 
     ### Move actions (e.g. go to next room) ###
 
-    def rotate_rooms_right(self, args=None):
+    def rotate_rooms_right(self, args=None) -> None:
         """
         rotate the rooms list to the right
         """
         self.tabs.next()
 
-    def rotate_rooms_left(self, args=None):
+    def rotate_rooms_left(self, args=None) -> None:
         """
         rotate the rooms list to the right
         """
         self.tabs.prev()
 
-    def go_to_room_number(self):
+    def go_to_room_number(self) -> None:
         """
         Read 2 more chars and go to the tab
         with the given number
         """
 
-        def read_next_digit(digit):
+        def read_next_digit(digit) -> None:
             try:
                 int(digit)
             except ValueError:
@@ -1007,22 +1115,50 @@ class Core:
 
         keyboard.continuation_keys_callback = read_next_digit
 
-    def go_to_roster(self):
+    def go_to_room_name(self) -> None:
+        room_name_jump = []
+
+        def read_next_letter(s) -> None:
+            nonlocal room_name_jump
+            room_name_jump.append(s)
+            any_matched, unique_tab = self.tabs.find_by_unique_prefix(
+                "".join(room_name_jump)
+            )
+
+            if not any_matched:
+                return
+
+            if unique_tab is not None:
+                self.tabs.set_current_tab(unique_tab)
+                # NOTE: returning here means that as soon as the tab is
+                # matched, normal input resumes. If we do *not* return here,
+                # any further characters matching the prefix of the tab will
+                # be swallowed (and a lot of tab switching will happen...),
+                # until a non-matching character or escape or something is
+                # pressed.
+                # This behaviour *may* be desirable.
+                return
+
+            keyboard.continuation_keys_callback = read_next_letter
+
+        keyboard.continuation_keys_callback = read_next_letter
+
+    def go_to_roster(self) -> None:
         "Select the roster as the current tab"
         self.tabs.set_current_tab(self.tabs.first())
 
-    def go_to_previous_tab(self):
+    def go_to_previous_tab(self) -> None:
         "Go to the previous tab"
         self.tabs.restore_previous_tab()
 
-    def go_to_important_room(self):
+    def go_to_important_room(self) -> None:
         """
         Go to the next room with activity, in the order defined in the
         dict tabs.STATE_PRIORITY
         """
         # shortcut
         priority = tabs.STATE_PRIORITY
-        tab_refs = {}
+        tab_refs = {}  # type: Dict[str, List[tabs.Tab]]
         # put all the active tabs in a dict of lists by state
         for tab in self.tabs.get_tabs():
             if not tab:
@@ -1071,7 +1207,7 @@ class Core:
         provided, we open a StaticConversationTab, else a
         DynamicConversationTab
         """
-        if safeJID(jid).resource:
+        if jid.resource:
             new_tab = tabs.StaticConversationTab(self, jid)
         else:
             new_tab = tabs.DynamicConversationTab(self, jid)
@@ -1082,7 +1218,7 @@ class Core:
         return new_tab
 
     def open_private_window(self, room_name: str, user_nick: str,
-                            focus=True) -> tabs.PrivateTab:
+                            focus=True) -> Optional[tabs.PrivateTab]:
         """
         Open a Private conversation in a MUC and focus if needed.
         """
@@ -1122,7 +1258,7 @@ class Core:
         return new_tab
 
     def open_new_form(self, form, on_cancel: Callable, on_send: Callable,
-                      **kwargs):
+                      **kwargs) -> None:
         """
         Open a new tab containing the form
         The callback are called with the completed form as parameter in
@@ -1133,9 +1269,9 @@ class Core:
 
     ### Modifying actions ###
 
-    def rename_private_tabs(self, room_name: str, old_nick: str, user: User):
+    def rename_private_tabs(self, room_name: str, old_nick: str, user: User) -> None:
         """
-        Call this method when someone changes his/her nick in a MUC,
+        Call this method when someone changes their nick in a MUC,
         this updates the name of all the opened private conversations
         with him/her
         """
@@ -1145,7 +1281,7 @@ class Core:
             tab.rename_user(old_nick, user)
 
     def on_user_left_private_conversation(self, room_name: str, user: User,
-                                          status_message: str):
+                                          status_message: str) -> None:
         """
         The user left the MUC: add a message in the associated
         private conversation
@@ -1155,7 +1291,7 @@ class Core:
         if tab:
             tab.user_left(status_message, user)
 
-    def on_user_rejoined_private_conversation(self, room_name: str, nick: str):
+    def on_user_rejoined_private_conversation(self, room_name: str, nick: str) -> None:
         """
         The user joined a MUC: add a message in the associated
         private conversation
@@ -1167,7 +1303,7 @@ class Core:
 
     def disable_private_tabs(self,
                              room_name: str,
-                             reason: Optional[str] = None):
+                             reason: Optional[str] = None) -> None:
         """
         Disable private tabs when leaving a room
         """
@@ -1178,7 +1314,7 @@ class Core:
                 tab.deactivate(reason=reason)
 
     def enable_private_tabs(self, room_name: str,
-                            reason: Optional[str] = None):
+                            reason: Optional[str] = None) -> None:
         """
         Enable private tabs when joining a room
         """
@@ -1188,18 +1324,18 @@ class Core:
             if tab.name.startswith(room_name):
                 tab.activate(reason=reason)
 
-    def on_user_changed_status_in_private(self, jid: JID, status: str):
+    def on_user_changed_status_in_private(self, jid: JID, status: str) -> None:
         tab = self.tabs.by_name_and_class(jid, tabs.ChatTab)
         if tab is not None:  # display the message in private
             tab.update_status(status)
 
-    def close_tab(self, tab: tabs.Tab = None):
+    def close_tab(self, to_close: tabs.Tab = None) -> None:
         """
         Close the given tab. If None, close the current one
         """
-        was_current = tab is None
-        if tab is None:
-            tab = self.tabs.current_tab
+        was_current = to_close is None
+        tab = to_close or self.tabs.current_tab
+
         if isinstance(tab, tabs.RosterInfoTab):
             return  # The tab 0 should NEVER be closed
         tab.on_close()
@@ -1216,26 +1352,26 @@ class Core:
                   gc.get_referrers(tab))
         del tab
 
-    def add_information_message_to_conversation_tab(self, jid: JID, msg: str):
+    def add_information_message_to_conversation_tab(self, jid: JID, msg: str) -> None:
         """
         Search for a ConversationTab with the given jid (full or bare),
         if yes, add the given message to it
         """
         tab = self.tabs.by_name_and_class(jid, tabs.ConversationTab)
         if tab is not None:
-            tab.add_message(msg, typ=2)
+            tab.add_message(InfoMessage(msg), typ=2)
             if self.tabs.current_tab is tab:
                 self.refresh_window()
 
 ####################### Curses and ui-related stuff ###########################
 
-    def doupdate(self):
+    def doupdate(self) -> None:
         "Do a curses update"
         if not self.running:
             return
         curses.doupdate()
 
-    def information(self, msg: str, typ=''):
+    def information(self, msg: str, typ: str = '') -> bool:
         """
         Displays an informational message in the "Info" buffer
         """
@@ -1255,7 +1391,12 @@ class Core:
         colors = get_theme().INFO_COLORS
         color = colors.get(typ.lower(), colors.get('default', None))
         nb_lines = self.information_buffer.add_message(
-            msg, nickname=typ, nick_color=color)
+            Message(
+                txt=msg,
+                nickname=typ,
+                nick_color=color
+            )
+        )
         popup_on = config.get('information_buffer_popup_on').split()
         if isinstance(self.tabs.current_tab, tabs.RosterInfoTab):
             self.refresh_window()
@@ -1268,7 +1409,7 @@ class Core:
                 self.tabs.current_tab.refresh_input()
         return True
 
-    def _init_curses(self, stdscr):
+    def _init_curses(self, stdscr) -> None:
         """
         ncurses initialization
         """
@@ -1284,7 +1425,7 @@ class Core:
         curses.ungetch(" ")  # H4X: without this, the screen is
         stdscr.getkey()  # erased on the first "getkey()"
 
-    def reset_curses(self):
+    def reset_curses(self) -> None:
         """
         Reset terminal capabilities to what they were before ncurses
         init
@@ -1294,7 +1435,7 @@ class Core:
         curses.curs_set(1)
         curses.endwin()
 
-    def refresh_window(self):
+    def refresh_window(self) -> None:
         """
         Refresh everything
         """
@@ -1304,7 +1445,7 @@ class Core:
         self.doupdate()
         curses.curs_set(nocursor)
 
-    def refresh_tab_win(self):
+    def refresh_tab_win(self) -> None:
         """
         Refresh the window containing the tab list
         """
@@ -1312,7 +1453,7 @@ class Core:
         self.refresh_input()
         self.doupdate()
 
-    def refresh_input(self):
+    def refresh_input(self) -> None:
         """
         Refresh the input if it exists
         """
@@ -1485,17 +1626,6 @@ class Core:
             self.tab_win.resize(1, tabs.Tab.width, tabs.Tab.height - 2, 0)
             self.left_tab_win = None
 
-    def add_message_to_text_buffer(self, buff, txt, nickname=None):
-        """
-        Add the message to the room if possible, else, add it to the Info window
-        (in the Info tab of the info window in the RosterTab)
-        """
-        if not buff:
-            self.information('Trying to add a message in no room: %s' % txt,
-                             'Error')
-            return
-        buff.add_message(txt, nickname=nickname)
-
     def full_screen_redraw(self):
         """
         Completely erase and redraw the screen
@@ -1614,6 +1744,12 @@ class Core:
             usage='<number or name>',
             shortdesc='Go to the specified room',
             completion=self.completion.win)
+        self.register_command(
+            'wup',
+            self.command.wup,
+            usage='<prefix>',
+            shortdesc='Go to the tab whose name uniquely starts with prefix',
+            completion=self.completion.win)
         self.commands['w'] = self.commands['win']
         self.register_command(
             'move_tab',
@@ -1673,6 +1809,48 @@ class Core:
             "currently using in this room (instead of default_nick).",
             shortdesc="Bookmark a room online.",
             completion=self.completion.bookmark)
+        self.register_command(
+            'accept',
+            self.command.accept,
+            usage='[jid]',
+            desc='Allow the provided JID (or the selected contact '
+            'in your roster), to see your presence.',
+            shortdesc='Allow a user your presence.',
+            completion=self.completion.roster_barejids)
+        self.register_command(
+            'add',
+            self.command.add,
+            usage='<jid>',
+            desc='Add the specified JID to your roster, ask them to'
+            ' allow you to see his presence, and allow them to'
+            ' see your presence.',
+            shortdesc='Add a user to your roster.')
+        self.register_command(
+            'deny',
+            self.command.deny,
+            usage='[jid]',
+            desc='Deny your presence to the provided JID (or the '
+            'selected contact in your roster), who is asking'
+            'you to be in their roster.',
+            shortdesc='Deny a user your presence.',
+            completion=self.completion.roster_barejids)
+        self.register_command(
+            'remove',
+            self.command.remove,
+            usage='[jid]',
+            desc='Remove the specified JID from your roster. This '
+            'will unsubscribe you from its presence, cancel '
+            'its subscription to yours, and remove the item '
+            'from your roster.',
+            shortdesc='Remove a user from your roster.',
+            completion=self.completion.remove)
+        self.register_command(
+            'reconnect',
+            self.command.command_reconnect,
+            usage="[reconnect]",
+            desc='Disconnect from the remote server if you are '
+            'currently connected and then connect to it again.',
+            shortdesc='Disconnect and reconnect to the server.')
         self.register_command(
             'set',
             self.command.set,
@@ -1788,6 +1966,13 @@ class Core:
             desc='Invite jid in room with reason.',
             shortdesc='Invite someone in a room.',
             completion=self.completion.invite)
+        self.register_command(
+            'impromptu',
+            self.command.impromptu,
+            usage='<jid> [jid ...]',
+            desc='Invite specified JIDs into a newly created room.',
+            shortdesc='Invite specified JIDs into newly created room.',
+            completion=self.completion.impromptu)
         self.register_command(
             'invitations',
             self.command.invitations,
@@ -1863,6 +2048,22 @@ class Core:
                 shortdesc='Send your gaming activity.',
                 completion=None)
 
+    def check_blocking(self, features):
+        if 'urn:xmpp:blocking' in features and not self.xmpp.anon:
+            self.register_command(
+                'block',
+                self.command.block,
+                usage='[jid]',
+                shortdesc='Prevent a JID from talking to you.',
+                completion=self.completion.block)
+            self.register_command(
+                'unblock',
+                self.command.unblock,
+                usage='[jid]',
+                shortdesc='Allow a JID to talk to you.',
+                completion=self.completion.unblock)
+
+            self.xmpp.del_event_handler('session_start', self.check_blocking)
 
 ####################### Random things to move #################################
 
@@ -1874,19 +2075,13 @@ class Core:
             tab = self.tabs.by_name_and_class(bm.jid, tabs.MucTab)
             nick = bm.nick if bm.nick else self.own_nick
             if not tab:
-                self.open_new_room(
+                tab = self.open_new_room(
                     bm.jid, nick, focus=False, password=bm.password)
             self.initial_joins.append(bm.jid)
             # do not join rooms that do not have autojoin
             # but display them anyway
             if bm.autojoin:
-                muc.join_groupchat(
-                    self,
-                    bm.jid,
-                    nick,
-                    passwd=bm.password,
-                    status=self.status.message,
-                    show=self.status.show)
+                tab.join()
 
     def check_bookmark_storage(self, features):
         private = 'jabber:iq:private' in features
@@ -1919,15 +2114,18 @@ class Core:
             return
         error_message = self.get_error_message(error)
         tab.add_message(
-            error_message,
-            highlight=True,
-            nickname='Error',
-            nick_color=get_theme().COLOR_ERROR_MSG,
-            typ=2)
+            Message(
+                error_message,
+                highlight=True,
+                nickname='Error',
+                nick_color=get_theme().COLOR_ERROR_MSG,
+            ),
+            typ=2,
+        )
         code = error['error']['code']
         if code == '401':
             msg = 'To provide a password in order to join the room, type "/join / password" (replace "password" by the real password)'
-            tab.add_message(msg, typ=2)
+            tab.add_message(InfoMessage(msg), typ=2)
         if code == '409':
             if config.get('alternative_nickname') != '':
                 if not tab.joined:
@@ -1936,8 +2134,12 @@ class Core:
             else:
                 if not tab.joined:
                     tab.add_message(
-                        'You can join the room with an other nick, by typing "/join /other_nick"',
-                        typ=2)
+                        InfoMessage(
+                            'You can join the room with another nick, '
+                            'by typing "/join /other_nick"'
+                        ),
+                        typ=2,
+                    )
         self.refresh_window()
 
 
