@@ -9,6 +9,8 @@ log = logging.getLogger(__name__)
 from poezio.windows.base_wins import Win
 from poezio.theming import get_theme, to_curses_attr
 
+from typing import Optional
+
 
 class HelpText(Win):
     """
@@ -17,13 +19,15 @@ class HelpText(Win):
     command mode.
     """
 
-    def __init__(self, text=''):
-        Win.__init__(self)
-        self.txt = text
+    __slots__ = ('txt')
 
-    def refresh(self, txt=None):
+    def __init__(self, text: str = '') -> None:
+        Win.__init__(self)
+        self.txt = text  # type: str
+
+    def refresh(self, txt: Optional[str] = None) -> None:
         log.debug('Refresh: %s', self.__class__.__name__)
-        if txt:
+        if txt is not None:
             self.txt = txt
         self._win.erase()
         self.addstr(0, 0, self.txt[:self.width - 1],
@@ -31,8 +35,8 @@ class HelpText(Win):
         self.finish_line(get_theme().COLOR_INFORMATION_BAR)
         self._refresh()
 
-    def do_command(self, key, raw=False):
+    def do_command(self, key, raw: bool = False) -> bool:
         return False
 
-    def on_delete(self):
+    def on_delete(self) -> None:
         return
